@@ -12,16 +12,20 @@ from bravado.exception import HTTPError
 from django.core.serializers.json import DjangoJSONEncoder
 from django.conf import settings
 from django.contrib.auth.models import User, Group
-from django.db import models, transaction
+from django.db import models
 from django.utils.translation import gettext_lazy as _
 from django.utils.timezone import now
 
+from esi.errors import TokenExpiredError, TokenInvalidError, TokenError
+from esi.models import Token
+
 from allianceauth.authentication.models import CharacterOwnership
 from allianceauth.eveonline.models import EveCorporationInfo
 from allianceauth.notifications import notify
 
-from esi.errors import TokenExpiredError, TokenInvalidError, TokenError
-from esi.models import Token
+from app_utils.datetime import DATETIME_FORMAT
+from app_utils.helpers import chunks
+from app_utils.logging import LoggerAddTag, make_logger_prefix
 
 from .. import __title__
 from ..app_settings import (
@@ -37,12 +41,11 @@ from ..app_settings import (
     STRUCTURES_STRUCTURE_SYNC_GRACE_MINUTES,
 )
 from .eveuniverse import EvePlanet, EveSolarSystem, EveType, EveUniverse
-from structures.helpers.esi_fetch import esi_fetch
-from structures.helpers.esi_fetch import esi_fetch_with_localization
+from ..helpers.esi_fetch import esi_fetch
+from ..helpers.esi_fetch import esi_fetch_with_localization
 from .structures import Structure
-from .notifications import EveEntity, Notification
+from .notifications import EveEntity, Notification, NotificationType
 
-from ..utils import LoggerAddTag, make_logger_prefix, chunks, DATETIME_FORMAT
 
 logger = LoggerAddTag(logging.getLogger(__name__), __title__)
 
@@ -54,10 +57,8 @@ class General(models.Model):
         managed = False
         default_permissions = ()
         permissions = (
-            (
-                "basic_access",
-                "Can access this app and view structures of your corporation",
-            ),
+            ("basic_access", "Can access this app and view public pages"),
+            ("view_corporation_structures", "Can view corporation structures"),
             ("view_alliance_structures", "Can view alliance structures"),
             ("view_all_structures", "Can view all structures"),
             ("add_structure_owner", "Can add new structure owner"),
@@ -282,26 +283,22 @@ class Owner(models.Model):
                         )
                     )
 
-                with transaction.atomic():
-                    # remove structures no longer returned from ESI
-                    ids_local = {x.id for x in Structure.objects.filter(owner=self)}
-                    ids_from_esi = {x["structure_id"] for x in structures}
-                    ids_to_remove = ids_local - ids_from_esi
-
-                    if len(ids_to_remove) > 0:
-                        Structure.objects.filter(id__in=ids_to_remove).delete()
-                        logger.info(
-                            "Removed {} structures which apparently no longer "
-                            "exist.".format(len(ids_to_remove))
-                        )
-
-                    # update structures
-                    for structure in structures:
-                        Structure.objects.update_or_create_from_dict(structure, self)
-
-                    self.structures_last_error = self.ERROR_NONE
-                    self.structures_last_sync = now()
-                    self.save()
+                # remove structures no longer returned from ESI
+                ids_local = {x.id for x in Structure.objects.filter(owner=self)}
+                ids_from_esi = {x["structure_id"] for x in structures}
+                ids_to_remove = ids_local - ids_from_esi
+                if len(ids_to_remove) > 0:
+                    Structure.objects.filter(id__in=ids_to_remove).delete()
+                    logger.info(
+                        "Removed {} structures which apparently no longer "
+                        "exist.".format(len(ids_to_remove))
+                    )
+                # update structures
+                for structure in structures:
+                    Structure.objects.update_or_create_from_dict(structure, self)
+                self.structures_last_error = self.ERROR_NONE
+                self.structures_last_sync = now()
+                self.save()
 
             except Exception as ex:
                 logger.exception(
@@ -704,9 +701,7 @@ class Owner(models.Model):
 
     def fetch_notifications_esi(self, user: User = None) -> bool:
         """fetches notification for the current owners and proceses them"""
-
-        add_prefix = self._logger_prefix()
-        notifications_count = 0
+        notifications_count_all = 0
         try:
             token, error = self.token()
             if error:
@@ -718,31 +713,25 @@ class Owner(models.Model):
             # fetch notifications from ESI
             try:
                 notifications = self._fetch_notifications_from_esi(token)
-                (
-                    new_notifications_count,
-                    notifications_count,
-                ) = self._store_notifications(notifications, notifications_count)
-                if new_notifications_count > 0:
+                notifications_count_new = self._store_notifications(notifications)
+                if notifications_count_new > 0:
                     logger.info(
-                        add_prefix(
-                            "Received {} new notifications from ESI".format(
-                                new_notifications_count
-                            )
-                        )
+                        "%s: Received %d new notifications from ESI",
+                        self,
+                        notifications_count_new,
                     )
                     self._process_timers_for_notifications(token)
+                    notifications_count_all += notifications_count_new
 
                 else:
-                    logger.info(add_prefix("No new notifications received from ESI"))
+                    logger.info("%s: No new notifications received from ESI", self)
 
                 self.notifications_last_error = self.ERROR_NONE
                 self.notifications_last_sync = now()
                 self.save()
 
             except Exception as ex:
-                logger.exception(
-                    add_prefix("An unexpected error ocurred {}".format(ex))
-                )
+                logger.exception("%s: An unexpected error ocurred %s", self, ex)
                 self.notifications_last_error = self.ERROR_UNKNOWN
                 self.notifications_last_sync = now()
                 self.save()
@@ -757,7 +746,7 @@ class Owner(models.Model):
 
         if user:
             self._send_report_to_user(
-                "notifications", notifications_count, success, error_code, user
+                "notifications", notifications_count_all, success, error_code, user
             )
 
         return success
@@ -775,31 +764,8 @@ class Owner(models.Model):
             self._store_raw_data(
                 "notifications", notifications, self.corporation.corporation_id
             )
-
         if STRUCTURES_NOTIFICATIONS_ARCHIVING_ENABLED:
-            # store notifications to disk in continuous file per corp
-            folder_name = "structures_notifications_archive"
-            os.makedirs(folder_name, exist_ok=True)
-            filename = "{}/notifications_{}_{}.txt".format(
-                folder_name, self.corporation.corporation_id, now().date().isoformat()
-            )
-            logger.info(
-                add_prefix(
-                    "Storing notifications into archive file: {}".format(filename)
-                )
-            )
-            with open(file=filename, mode="a", encoding="utf-8") as f:
-                f.write(
-                    "[{}] {}:\n".format(
-                        now().strftime(DATETIME_FORMAT),
-                        self.corporation.corporation_ticker,
-                    )
-                )
-                json.dump(
-                    notifications, f, cls=DjangoJSONEncoder, sort_keys=True, indent=4
-                )
-                f.write("\n")
-
+            self._store_raw_notifications(notifications)
         logger.debug(
             add_prefix(
                 "Processing {:,} notifications received from ESI".format(
@@ -809,53 +775,76 @@ class Owner(models.Model):
         )
         return notifications
 
-    def _store_notifications(self, notifications, notifications_count):
-        """stores all notifications in database"""
-        new_notifications_count = 0
-        with transaction.atomic():
-            for notification in notifications:
-                notification_type = Notification.get_matching_notification_type(
-                    notification["type"]
+    def _store_raw_notifications(self, notifications):
+        # store notifications to disk in continuous file per corp
+        folder_name = "structures_notifications_archive"
+        os.makedirs(folder_name, exist_ok=True)
+        filename = "{}/notifications_{}_{}.txt".format(
+            folder_name, self.corporation.corporation_id, now().date().isoformat()
+        )
+        logger.info(
+            "%s: Storing notifications into archive file: %s", self, format(filename)
+        )
+        with open(file=filename, mode="a", encoding="utf-8") as f:
+            f.write(
+                "[{}] {}:\n".format(
+                    now().strftime(DATETIME_FORMAT),
+                    self.corporation.corporation_ticker,
                 )
-                if notification_type:
-                    sender_type = EveEntity.get_matching_entity_category(
-                        notification["sender_type"]
-                    )
-                    if sender_type != EveEntity.CATEGORY_OTHER:
-                        sender, _ = EveEntity.objects.get_or_create_esi(
-                            notification["sender_id"]
-                        )
-                    else:
-                        sender, _ = EveEntity.objects.get_or_create(
-                            id=notification["sender_id"],
-                            defaults={"category": sender_type},
-                        )
-                    text = notification["text"] if "text" in notification else None
-                    is_read = (
-                        notification["is_read"] if "is_read" in notification else None
-                    )
-                    obj, created = Notification.objects.update_or_create(
-                        notification_id=notification["notification_id"],
-                        owner=self,
-                        defaults={
-                            "sender": sender,
-                            "timestamp": notification["timestamp"],
-                            "notification_type": notification_type,
-                            "text": text,
-                            "is_read": is_read,
-                            "last_updated": now(),
-                        },
-                    )
-                    notifications_count += 1
-                    if created:
-                        obj.created = now()
-                        obj.save()
-                        new_notifications_count += 1
+            )
+            json.dump(notifications, f, cls=DjangoJSONEncoder, sort_keys=True, indent=4)
+            f.write("\n")
 
-            self.notifications_last_error = self.ERROR_NONE
-            self.save()
+    def _store_notifications(self, notifications: list) -> int:
+        """stores new notifications in database.
+        Returns number of newly created objects.
+        """
+        # identify new notifications
+        existing_notification_ids = set(
+            self.notification_set.values_list("notification_id", flat=True)
+        )
+        new_notifications = [
+            obj
+            for obj in notifications
+            if obj["notification_id"] not in existing_notification_ids
+        ]
+        # create new notif objects
+        new_notification_objects = list()
+        for notification in new_notifications:
+            sender_type = EveEntity.get_matching_entity_category(
+                notification["sender_type"]
+            )
+            if sender_type != EveEntity.CATEGORY_OTHER:
+                sender, _ = EveEntity.objects.get_or_create_esi(
+                    eve_entity_id=notification["sender_id"]
+                )
+            else:
+                sender, _ = EveEntity.objects.get_or_create(
+                    id=notification["sender_id"],
+                    defaults={"category": sender_type},
+                )
+            text = notification["text"] if "text" in notification else None
+            is_read = notification["is_read"] if "is_read" in notification else None
+            new_notification_objects.append(
+                Notification(
+                    notification_id=notification["notification_id"],
+                    owner=self,
+                    sender=sender,
+                    timestamp=notification["timestamp"],
+                    # at least one type has a trailing white space
+                    # which we need to remove
+                    notif_type=notification["type"].strip(),
+                    text=text,
+                    is_read=is_read,
+                    last_updated=now(),
+                    created=now(),
+                )
+            )
 
-        return new_notifications_count, notifications_count
+        Notification.objects.bulk_create(new_notification_objects)
+        self.notifications_last_error = self.ERROR_NONE
+        self.save()
+        return len(new_notification_objects)
 
     def _process_timers_for_notifications(self, token: Token):
         """processes notifications for timers if any"""
@@ -863,10 +852,9 @@ class Owner(models.Model):
             cutoff_dt_for_stale = now() - timedelta(
                 hours=STRUCTURES_HOURS_UNTIL_STALE_NOTIFICATION
             )
-            my_types = Notification.get_types_for_timerboard()
             notifications = (
                 Notification.objects.filter(owner=self)
-                .filter(notification_type__in=my_types)
+                .filter(notif_type__in=NotificationType.relevant_for_timerboard)
                 .exclude(is_timer_added=True)
                 .filter(timestamp__gte=cutoff_dt_for_stale)
                 .select_related()
@@ -892,6 +880,7 @@ class Owner(models.Model):
                 )
                 all_new_notifications = list(
                     Notification.objects.filter(owner=self)
+                    .filter(notif_type__in=NotificationType.values)
                     .filter(is_sent=False)
                     .filter(timestamp__gte=cutoff_dt_for_stale)
                     .select_related()
@@ -902,9 +891,9 @@ class Owner(models.Model):
                 for webhook in self.webhooks.filter(is_active=True):
                     active_webhooks_count += 1
                     new_notifications = [
-                        x
-                        for x in all_new_notifications
-                        if str(x.notification_type) in webhook.notification_types
+                        notif
+                        for notif in all_new_notifications
+                        if str(notif.notif_type) in webhook.notification_types
                     ]
                     if len(new_notifications) > 0:
                         new_notifications_count += len(new_notifications)
