@@ -3,7 +3,7 @@ from django.db import models
 from allianceauth.services.hooks import get_extension_logger
 
 from .. import __title__
-from ..utils import LoggerAddTag
+from app_utils.logging import LoggerAddTag
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 
